import bpy
from bpy.props import IntProperty, BoolProperty, FloatProperty, FloatVectorProperty, StringProperty, EnumProperty
from .import __package__ as base_package
from .utils import draw_keymap
from .add_menu import update_add_menu
from .utils import keymap_items

def make_pie_menu_active(self, context):
    wm = bpy.context.window_manager
    keyconfig = wm.keyconfigs.user
    km = keyconfig.keymaps['Object Mode']
    kmi = km.keymap_items['wm.call_menu_pie']
    kmi.active = self.pie_menu

# TODO: 
"""
The object attributes should be hidden, like edit mesh on export!

Later:
Profile, slower than regular key ops add? Maybe deafult?? Profile!
Add scale supprot to custom??
Custom .blend?
Option for auto smooth? Quad sphere should at least be smooth
Name is too long, ND primitives?
"""

CONST_ITEMS = [("Grid", "MESH_PLANE"),
                ("Cube", "MESH_CUBE"),
                ("Circle", "MESH_CIRCLE"),
                ("UV Sphere", "MESH_UVSPHERE"),
                ("Ico Sphere", "MESH_ICOSPHERE"),
                ("Cylinder", "MESH_CYLINDER"),
                ("Cone", "MESH_CONE"),
                ("Torus", "MESH_TORUS"),
                ("Quad Sphere", "SPHERE"),
                ("Capsule", "MESH_CAPSULE"),
                ("Mod Cylinder", "MOD_SCREW"),
                ("Spiral", "MOD_SCREW"),
                ("Other", "ADD"),
                # (".blend", "BLENDER"),
                ("CUSTOM_OP", "FILE_SCRIPT")]

add_pie_menu_items = [
    (f"{item[0]}", item[0], "", item[1], index)
    for index, item in enumerate(CONST_ITEMS)]

class Non_Destructive_Primitives_Preferences(bpy.types.AddonPreferences):
    bl_idname = base_package 

    pie_menu: BoolProperty(name="Pie Menu", default=True, update=make_pie_menu_active) # type: ignore
    show_in_menu: EnumProperty(name="ND Primitives in", items=[("ADD_MESH", "Add Mesh Menu", ""),
                                                           ("OWN_CATEGORY_IN_ADD_MENU", "Menu in Add Mesh", ""),
                                                           ("OWN_MENU_CATEGORY", "ND Menu in Add Object Menu", ""),
                                                           ("NONE", "None", "")], default="ADD_MESH", update=update_add_menu)# type: ignore
    # use_relative_scale: BoolProperty(name="Auto Scale", default=True, description="Default to Relative Scale Value") # type: ignore
    # default_scale: FloatProperty(name="Default Size", default=1.0, min=0.1, max=10.0, unit='LENGTH', description="Default Regular Scale Size Value") # type: ignore
    min_scale: FloatProperty(name="Min Size", default=0.01, min=0.0001, max=10.0, unit='LENGTH', description="Auto Scale Min Size Value") # type: ignore
    delete_modifier: BoolProperty(name="Delete Modifier when entering Edit Mode", default=False, description="Default to Relative Scale Value") # type: ignore
    auto_swith_to_modifier_tab: BoolProperty(name="Auto Switch to Modifier Tab on Create", default=True) # type: ignore
    show_gizmo: BoolProperty(name="Show Gizmo", default=True) # type: ignore

    pie_add_left: EnumProperty(name="Left", items=add_pie_menu_items, default="UV Sphere") # type: ignore
    pie_add_right: EnumProperty(name="Right", items=add_pie_menu_items, default="Cube") # type: ignore
    pie_add_bottom: EnumProperty(name="Bottom", items=add_pie_menu_items, default="Cylinder") # type: ignore
    pie_add_top: EnumProperty(name="Top", items=add_pie_menu_items, default="Grid") # type: ignore
    pie_add_top_left: EnumProperty(name="Top Left", items=add_pie_menu_items, default="Mod Cylinder") # type: ignore
    pie_add_top_right: EnumProperty(name="Top Right", items=add_pie_menu_items, default="Other") # type: ignore
    pie_add_bottom_left: EnumProperty(name="Bottom Left", items=add_pie_menu_items, default="Quad Sphere") # type: ignore
    pie_add_bottom_right: EnumProperty(name="Bottom Right", items=add_pie_menu_items, default="Circle") # type: ignore
    
    pie_add_custom_op_left: StringProperty(default="") # type: ignore
    pie_add_custom_op_name_left: StringProperty(default="") # type: ignore

    pie_add_custom_op_right: StringProperty(default="") # type: ignore
    pie_add_custom_op_name_right: StringProperty(default="") # type: ignore

    pie_add_custom_op_top: StringProperty(default="") # type: ignore
    pie_add_custom_op_name_top: StringProperty(default="") # type: ignore

    pie_add_custom_op_bottom: StringProperty(default="") # type: ignore
    pie_add_custom_op_name_bottom: StringProperty(default="") # type: ignore

    pie_add_custom_op_top_left: StringProperty(default="") # type: ignore
    pie_add_custom_op_name_top_left: StringProperty(default="") # type: ignore

    pie_add_custom_op_top_right: StringProperty(default="") # type: ignore
    pie_add_custom_op_name_top_right: StringProperty(default="") # type: ignore

    pie_add_custom_op_bottom_left: StringProperty(default="") # type: ignore
    pie_add_custom_op_name_bottom_left: StringProperty(default="") # type: ignore

    pie_add_custom_op_bottom_right: StringProperty(default="") # type: ignore
    pie_add_custom_op_name_bottom_right: StringProperty(default="") # type: ignore
    def draw(self, context):
        layout = self.layout
        # row.prop(self,"default_scale")
        row = layout.row()
        row.prop(self,"delete_modifier")
        row = layout.row()
        row.prop(self,"auto_swith_to_modifier_tab")
        row = layout.row()
        row.alignment = 'LEFT'
        row.label(text="Auto Scale:")
        row.prop(self,"min_scale")
        row.prop(self,"show_gizmo")
        row = layout.row()

        # # menu
        row = layout.row()
        row.alignment = 'EXPAND'
        row.prop(self,"show_in_menu")
        row = layout.row()
        row = layout.row()

        # pie menu custimaztion
        row.prop(self,"pie_menu")
        if self.pie_menu:
            row = layout.row()
            row.label(text="Pie Menu:")
            pie_positions = [
                ("pie_add_left", "pie_add_custom_op_left", "pie_add_custom_op_name_left"),
                ("pie_add_right", "pie_add_custom_op_right", "pie_add_custom_op_name_right"),
                ("pie_add_bottom", "pie_add_custom_op_bottom", "pie_add_custom_op_name_bottom"),
                ("pie_add_top", "pie_add_custom_op_top", "pie_add_custom_op_name_top"),
                ("pie_add_top_left", "pie_add_custom_op_top_left", "pie_add_custom_op_name_top_left"),
                ("pie_add_top_right", "pie_add_custom_op_top_right", "pie_add_custom_op_name_top_right"),
                ("pie_add_bottom_left", "pie_add_custom_op_bottom_left", "pie_add_custom_op_name_bottom_left"),
                ("pie_add_bottom_right", "pie_add_custom_op_bottom_right", "pie_add_custom_op_name_bottom_right")
            ]

            for position, custom_op, custom_op_name in pie_positions:
                layout.prop(self, position)
                if getattr(self, position) == "CUSTOM_OP":
                    layout.prop(self, custom_op, text="Custom Operator")
                    layout.prop(self, custom_op_name, text="Name")

            # keymap
            row = layout.row()
            row = layout.row()
            row = layout.row()
            row.label(text="Pie Keymap:")
            draw_keymap(self, context)

